@@ -1,13 +1,14 @@
 //! Serde `Deserializer` module
 
 use crate::{
-    de::escape::EscapedDeserializer,
-    de::seq::{not_in, TagFilter},
+    de::key::QNameDeserializer,
     de::simple_type::SimpleTypeDeserializer,
-    de::{str2bool, DeEvent, Deserializer, XmlRead, INNER_VALUE, UNFLATTEN_PREFIX},
+    de::{str2bool, DeEvent, Deserializer, XmlRead, TEXT_KEY, VALUE_KEY},
+    encoding::Decoder,
     errors::serialize::DeError,
     events::attributes::IterState,
     events::BytesStart,
+    name::QName,
 };
 use serde::de::{self, DeserializeSeed, IntoDeserializer, SeqAccess, Visitor};
 use serde::serde_if_integer128;
@@ -49,19 +50,19 @@ enum ValueSource {
     Text,
     /// Next value should be deserialized from an element with an any name, except
     /// elements with a name matching one of the struct fields. Corresponding tag
-    /// name will always be associated with a field with name [`INNER_VALUE`].
+    /// name will always be associated with a field with name [`VALUE_KEY`].
     ///
     /// That state is set when call to [`peek()`] returns a [`Start`] event, which
     /// [`name()`] is not listed in the [list of known fields] (which for a struct
     /// is a list of field names, and for a map that is an empty list), _and_
-    /// struct has a field with a special name [`INNER_VALUE`].
+    /// struct has a field with a special name [`VALUE_KEY`].
     ///
     /// When in this state, next event, returned by [`next()`], will be a [`Start`],
     /// which represents both a key, and a value. Value would be deserialized from
     /// the whole element and how is will be done determined by the value deserializer.
     /// The [`MapAccess`] do not consume any events in that state.
     ///
-    /// Because in that state any encountered `<tag>` is mapped to the [`INNER_VALUE`]
+    /// Because in that state any encountered `<tag>` is mapped to the [`VALUE_KEY`]
     /// field, it is possible to use tag name as an enum discriminator, so `enum`s
     /// can be deserialized from that XMLs:
     ///
@@ -97,9 +98,6 @@ enum ValueSource {
     /// of a `<variant1>...</variant1>` or `<variant2>...</variant2>` node, including
     /// the tag name.
     ///
-    /// Currently, processing of that enum variant is fully equivalent to the
-    /// processing of a [`Text`] variant. Split of variants made for clarity.
-    ///
     /// [`Start`]: DeEvent::Start
     /// [`peek()`]: Deserializer::peek()
     /// [`next()`]: Deserializer::next()
@@ -136,7 +134,7 @@ enum ValueSource {
     /// variant based on the tag name. If that is needed, then [`Content`] variant
     /// of this enum should be used. Such usage is enabled by annotating a struct
     /// field as "content" field, which implemented as given the field a special
-    /// [`INNER_VALUE`] name.
+    /// [`VALUE_KEY`] name.
     ///
     /// [`Start`]: DeEvent::Start
     /// [`peek()`]: Deserializer::peek()
@@ -146,6 +144,8 @@ enum ValueSource {
     Nested,
 }
 
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
 /// A deserializer that extracts map-like structures from an XML. This deserializer
 /// represents a one XML tag:
 ///
@@ -180,15 +180,13 @@ where
     /// List of field names of the struct. It is empty for maps
     fields: &'static [&'static str],
     /// If `true`, then the deserialized struct has a field with a special name:
-    /// [`INNER_VALUE`]. That field should be deserialized from the text content
-    /// of an XML node:
+    /// [`VALUE_KEY`]. That field should be deserialized from the whole content
+    /// of an XML node, including tag name:
     ///
     /// ```xml
-    /// <tag>value for INNER_VALUE field<tag>
+    /// <tag>value for VALUE_KEY field<tag>
     /// ```
     has_value_field: bool,
-    /// list of fields yet to unflatten (defined as starting with $unflatten=)
-    unflatten_fields: Vec<&'static [u8]>,
 }
 
 impl<'de, 'a, R> MapAccess<'de, 'a, R>
@@ -207,12 +205,7 @@ where
             start,
             source: ValueSource::Unknown,
             fields,
-            has_value_field: fields.contains(&INNER_VALUE),
-            unflatten_fields: fields
-                .iter()
-                .filter(|f| f.starts_with(UNFLATTEN_PREFIX))
-                .map(|f| f.as_bytes())
-                .collect(),
+            has_value_field: fields.contains(&VALUE_KEY),
         })
     }
 }
@@ -237,21 +230,28 @@ where
             // try getting map from attributes (key= "value")
             let (key, value) = a.into();
             self.source = ValueSource::Attribute(value.unwrap_or_default());
-            seed.deserialize(EscapedDeserializer::new(
-                Cow::Borrowed(&slice[key]),
-                decoder,
-                false,
-            ))
-            .map(Some)
+
+            let de = QNameDeserializer::from_attr(QName(&slice[key]), decoder)?;
+            seed.deserialize(de).map(Some)
         } else {
             // try getting from events (<key>value</key>)
             match self.de.peek()? {
+                // We shouldn't have both `$value` and `$text` fields in the same
+                // struct, so if we have `$value` field, the we should deserialize
+                // text content to `$value`
+                DeEvent::Text(_) | DeEvent::CData(_) if self.has_value_field => {
+                    self.source = ValueSource::Content;
+                    // Deserialize `key` from special attribute name which means
+                    // that value should be taken from the text content of the
+                    // XML node
+                    seed.deserialize(VALUE_KEY.into_deserializer()).map(Some)
+                }
                 DeEvent::Text(_) | DeEvent::CData(_) => {
                     self.source = ValueSource::Text;
                     // Deserialize `key` from special attribute name which means
                     // that value should be taken from the text content of the
                     // XML node
-                    seed.deserialize(INNER_VALUE.into_deserializer()).map(Some)
+                    seed.deserialize(TEXT_KEY.into_deserializer()).map(Some)
                 }
                 // Used to deserialize collections of enums, like:
                 // <root>
@@ -271,32 +271,13 @@ where
                 // See https://github.com/serde-rs/serde/issues/1905
                 DeEvent::Start(e) if self.has_value_field && not_in(self.fields, e, decoder)? => {
                     self.source = ValueSource::Content;
-                    seed.deserialize(INNER_VALUE.into_deserializer()).map(Some)
+                    seed.deserialize(VALUE_KEY.into_deserializer()).map(Some)
                 }
                 DeEvent::Start(e) => {
                     self.source = ValueSource::Nested;
-                    let key = if let Some(p) = self
-                        .unflatten_fields
-                        .iter()
-                        .position(|f| e.name().as_ref() == &f[UNFLATTEN_PREFIX.len()..])
-                    {
-                        // Used to deserialize elements, like:
-                        // <root>
-                        //   <xxx>test</xxx>
-                        // </root>
-                        //
-                        // into
-                        //
-                        // struct Root {
-                        //     #[serde(rename = "$unflatten=xxx")]
-                        //     xxx: String,
-                        // }
-                        seed.deserialize(self.unflatten_fields.remove(p).into_deserializer())
-                    } else {
-                        let name = Cow::Borrowed(e.local_name().into_inner());
-                        seed.deserialize(EscapedDeserializer::new(name, decoder, false))
-                    };
-                    key.map(Some)
+
+                    let de = QNameDeserializer::from_elem(e.name(), decoder)?;
+                    seed.deserialize(de).map(Some)
                 }
                 // Stop iteration after reaching a closing tag
                 DeEvent::End(e) if e.name() == self.start.name() => Ok(None),
@@ -325,20 +306,18 @@ where
             //   text value
             // </any-tag>
             // The whole map represented by an `<any-tag>` element, the map key
-            // is implicit and equals to the `INNER_VALUE` constant, and the value
+            // is implicit and equals to the `TEXT_KEY` constant, and the value
             // is a `Text` or a `CData` event (the value deserializer will see one
             // of that events)
             // This case are checked by "xml_schema_lists::element" tests in tests/serde-de.rs
             ValueSource::Text => match self.de.next()? {
-                DeEvent::Text(e) => seed.deserialize(SimpleTypeDeserializer::from_cow(
-                    e.into_inner(),
-                    true,
-                    self.de.reader.decoder(),
+                DeEvent::Text(e) => seed.deserialize(SimpleTypeDeserializer::from_text_content(
+                    // Comment to prevent auto-formatting
+                    e.decode(true)?,
                 )),
-                DeEvent::CData(e) => seed.deserialize(SimpleTypeDeserializer::from_cow(
-                    e.into_inner(),
-                    false,
-                    self.de.reader.decoder(),
+                DeEvent::CData(e) => seed.deserialize(SimpleTypeDeserializer::from_text_content(
+                    // Comment to prevent auto-formatting
+                    e.decode()?,
                 )),
                 // SAFETY: We set `Text` only when we seen `Text` or `CData`
                 _ => unreachable!(),
@@ -348,7 +327,7 @@ where
             //   <any>...</any>
             // </any-tag>
             // The whole map represented by an `<any-tag>` element, the map key
-            // is implicit and equals to the `INNER_VALUE` constant, and the value
+            // is implicit and equals to the `VALUE_KEY` constant, and the value
             // is a `Start` event (the value deserializer will see that event)
             ValueSource::Content => seed.deserialize(MapValueDeserializer {
                 map: self,
@@ -400,7 +379,7 @@ where
     /// Access to the map that created this deserializer. Gives access to the
     /// context, such as list of fields, that current map known about.
     map: &'m mut MapAccess<'de, 'a, R>,
-    /// Determines, should [`Deserializer::next_text_impl()`] expand the second
+    /// Determines, should [`Deserializer::read_string_impl()`] expand the second
     /// level of tags or not.
     ///
     /// If this field is `true`, we process the following XML shape:
@@ -451,7 +430,7 @@ where
     /// ```
     ///
     /// The whole map represented by an `<any-tag>` element, the map key is
-    /// implicit and equals to the [`INNER_VALUE`] constant, and the value is
+    /// implicit and equals to the [`VALUE_KEY`] constant, and the value is
     /// a [`Text`], a [`CData`], or a [`Start`] event (the value deserializer
     /// will see one of those events). In the first two cases the value of this
     /// field do not matter (because we already see the textual event and there
@@ -461,7 +440,7 @@ where
     ///
     /// ```ignore
     /// struct AnyName {
-    ///   #[serde(rename = "$value")]
+    ///   #[serde(rename = "$text")]
     ///   any_name: String,
     /// }
     /// ```
@@ -470,7 +449,7 @@ where
     /// Changing this can be valuable for <https://github.com/tafia/quick-xml/issues/383>,
     /// but those fields should be explicitly marked that they want to get any
     /// possible markup as a `String` and that mark is different from marking them
-    /// as accepting "text content" which the currently `$value` means.
+    /// as accepting "text content" which the currently `$text` means.
     ///
     /// [`Text`]: DeEvent::Text
     /// [`CData`]: DeEvent::CData
@@ -482,10 +461,14 @@ impl<'de, 'a, 'm, R> MapValueDeserializer<'de, 'a, 'm, R>
 where
     R: XmlRead<'de>,
 {
-    /// Returns a text event, used inside [`deserialize_primitives!()`]
+    /// Returns a next string as concatenated content of consequent [`Text`] and
+    /// [`CData`] events, used inside [`deserialize_primitives!()`].
+    ///
+    /// [`Text`]: DeEvent::Text
+    /// [`CData`]: DeEvent::CData
     #[inline]
-    fn next_text(&mut self, unescape: bool) -> Result<Cow<'de, str>, DeError> {
-        self.map.de.next_text_impl(unescape, self.allow_start)
+    fn read_string(&mut self, unescape: bool) -> Result<Cow<'de, str>, DeError> {
+        self.map.de.read_string_impl(unescape, self.allow_start)
     }
 }
 
@@ -537,6 +520,16 @@ where
         self.deserialize_tuple(len, visitor)
     }
 
+    /// Deserializes each `<tag>` in
+    /// ```xml
+    /// <any-tag>
+    ///   <tag>...</tag>
+    ///   <tag>...</tag>
+    ///   <tag>...</tag>
+    /// </any-tag>
+    /// ```
+    /// as a sequence item, where `<any-tag>` represents a Map in a [`Self::map`],
+    /// and a `<tag>` is a sequential field of that map.
     fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
@@ -570,9 +563,93 @@ where
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
+/// Check if tag `start` is included in the `fields` list. `decoder` is used to
+/// get a string representation of a tag.
+///
+/// Returns `true`, if `start` is not in the `fields` list and `false` otherwise.
+fn not_in(
+    fields: &'static [&'static str],
+    start: &BytesStart,
+    decoder: Decoder,
+) -> Result<bool, DeError> {
+    let tag = decoder.decode(start.name().into_inner())?;
+
+    Ok(fields.iter().all(|&field| field != tag.as_ref()))
+}
+
+/// A filter that determines, what tags should form a sequence.
+///
+/// There are two types of sequences:
+/// - sequence where each element represented by tags with the same name
+/// - sequence where each element can have a different tag
+///
+/// The first variant could represent a collection of structs, the second --
+/// a collection of enum variants.
+///
+/// In the second case we don't know what tag name should be expected as a
+/// sequence element, so we accept any element. Since the sequence are flattened
+/// into maps, we skip elements which have dedicated fields in a struct by using an
+/// `Exclude` filter that filters out elements with names matching field names
+/// from the struct.
+///
+/// # Lifetimes
+///
+/// `'de` represents a lifetime of the XML input, when filter stores the
+/// dedicated tag name
+#[derive(Debug)]
+enum TagFilter<'de> {
+    /// A `SeqAccess` interested only in tags with specified name to deserialize
+    /// an XML like this:
+    ///
+    /// ```xml
+    /// <...>
+    ///   <tag/>
+    ///   <tag/>
+    ///   <tag/>
+    ///   ...
+    /// </...>
+    /// ```
+    ///
+    /// The tag name is stored inside (`b"tag"` for that example)
+    Include(BytesStart<'de>), //TODO: Need to store only name instead of a whole tag
+    /// A `SeqAccess` interested in tags with any name, except explicitly listed.
+    /// Excluded tags are used as struct field names and therefore should not
+    /// fall into a `$value` category
+    Exclude(&'static [&'static str]),
+}
+
+impl<'de> TagFilter<'de> {
+    fn is_suitable(&self, start: &BytesStart, decoder: Decoder) -> Result<bool, DeError> {
+        match self {
+            Self::Include(n) => Ok(n.name() == start.name()),
+            Self::Exclude(fields) => not_in(fields, start, decoder),
+        }
+    }
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
 /// An accessor to sequence elements forming a value for struct field.
 /// Technically, this sequence is flattened out into structure and sequence
-/// elements are overlapped with other fields of a structure
+/// elements are overlapped with other fields of a structure. Each call to
+/// [`Self::next_element_seed`] consumes a next sub-tree or consequent list
+/// of [`Text`] and [`CData`] events.
+///
+/// ```xml
+/// <>
+///   ...
+///   <item>The is the one item</item>
+///   This is <![CDATA[one another]]> item<!-- even when--> it splitted by comments
+///   <tag>...and that is the third!</tag>
+///   ...
+/// </>
+/// ```
+///
+/// Depending on [`Self::filter`], only some of that possible constructs would be
+/// an element.
+///
+/// [`Text`]: DeEvent::Text
+/// [`CData`]: DeEvent::CData
 struct MapValueSeqAccess<'de, 'a, 'm, R>
 where
     R: XmlRead<'de>,
@@ -639,7 +716,7 @@ where
 
                 // Start(tag), Text, CData
                 _ => seed
-                    .deserialize(SeqValueDeserializer { map: self.map })
+                    .deserialize(SeqItemDeserializer { map: self.map })
                     .map(Some),
             };
         }
@@ -648,8 +725,8 @@ where
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-/// A deserializer for a value of sequence.
-struct SeqValueDeserializer<'de, 'a, 'm, R>
+/// A deserializer for a single item of a sequence.
+struct SeqItemDeserializer<'de, 'a, 'm, R>
 where
     R: XmlRead<'de>,
 {
@@ -658,18 +735,22 @@ where
     map: &'m mut MapAccess<'de, 'a, R>,
 }
 
-impl<'de, 'a, 'm, R> SeqValueDeserializer<'de, 'a, 'm, R>
+impl<'de, 'a, 'm, R> SeqItemDeserializer<'de, 'a, 'm, R>
 where
     R: XmlRead<'de>,
 {
-    /// Returns a text event, used inside [`deserialize_primitives!()`]
+    /// Returns a next string as concatenated content of consequent [`Text`] and
+    /// [`CData`] events, used inside [`deserialize_primitives!()`].
+    ///
+    /// [`Text`]: DeEvent::Text
+    /// [`CData`]: DeEvent::CData
     #[inline]
-    fn next_text(&mut self, unescape: bool) -> Result<Cow<'de, str>, DeError> {
-        self.map.de.next_text_impl(unescape, true)
+    fn read_string(&mut self, unescape: bool) -> Result<Cow<'de, str>, DeError> {
+        self.map.de.read_string_impl(unescape, true)
     }
 }
 
-impl<'de, 'a, 'm, R> de::Deserializer<'de> for SeqValueDeserializer<'de, 'a, 'm, R>
+impl<'de, 'a, 'm, R> de::Deserializer<'de> for SeqItemDeserializer<'de, 'a, 'm, R>
 where
     R: XmlRead<'de>,
 {
@@ -717,22 +798,31 @@ where
         self.deserialize_tuple(len, visitor)
     }
 
+    /// This method deserializes a sequence inside of element that itself is a
+    /// sequence element:
+    ///
+    /// ```xml
+    /// <>
+    ///   ...
+    ///   <self>inner sequence</self>
+    ///   <self>inner sequence</self>
+    ///   <self>inner sequence</self>
+    ///   ...
+    /// </>
+    /// ```
     fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
     {
         match self.map.de.next()? {
-            DeEvent::Text(e) => SimpleTypeDeserializer::from_cow(
-                // Comment to prevent auto-formatting and keep Text and Cdata similar
-                e.into_inner(),
-                true,
-                self.map.de.reader.decoder(),
+            DeEvent::Text(e) => SimpleTypeDeserializer::from_text_content(
+                // Comment to prevent auto-formatting
+                e.decode(true)?,
             )
             .deserialize_seq(visitor),
-            DeEvent::CData(e) => SimpleTypeDeserializer::from_cow(
-                e.into_inner(),
-                false,
-                self.map.de.reader.decoder(),
+            DeEvent::CData(e) => SimpleTypeDeserializer::from_text_content(
+                // Comment to prevent auto-formatting
+                e.decode()?,
             )
             .deserialize_seq(visitor),
             // This is a sequence element. We cannot treat it as another flatten
@@ -740,16 +830,14 @@ where
             // it to `xs:simpleType` implementation
             DeEvent::Start(e) => {
                 let value = match self.map.de.next()? {
-                    DeEvent::Text(e) => SimpleTypeDeserializer::from_cow(
-                        e.into_inner(),
-                        true,
-                        self.map.de.reader.decoder(),
+                    DeEvent::Text(e) => SimpleTypeDeserializer::from_text_content(
+                        // Comment to prevent auto-formatting
+                        e.decode(true)?,
                     )
                     .deserialize_seq(visitor),
-                    DeEvent::CData(e) => SimpleTypeDeserializer::from_cow(
-                        e.into_inner(),
-                        false,
-                        self.map.de.reader.decoder(),
+                    DeEvent::CData(e) => SimpleTypeDeserializer::from_text_content(
+                        // Comment to prevent auto-formatting
+                        e.decode()?,
                     )
                     .deserialize_seq(visitor),
                     e => Err(DeError::Unsupported(
@@ -771,3 +859,20 @@ where
         self.map.de.is_human_readable()
     }
 }
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
+#[test]
+fn test_not_in() {
+    let tag = BytesStart::new("tag");
+
+    assert_eq!(not_in(&[], &tag, Decoder::utf8()).unwrap(), true);
+    assert_eq!(
+        not_in(&["no", "such", "tags"], &tag, Decoder::utf8()).unwrap(),
+        true
+    );
+    assert_eq!(
+        not_in(&["some", "tag", "included"], &tag, Decoder::utf8()).unwrap(),
+        false
+    );
+}
